<?php

require_once(dirname(__FILE__).'/../../DesignerTabSelf.php');

class AdminAjaxController extends ModuleAdminController
{
    private $_required_php_version;
    private $_php_version;

    public function __construct()
    {
        $this->bootstrap = true;
        $this->display = 'view';
        $this->meta_title = $this->l('Billion Themler');
        $this->_php_version = phpversion();
        $this->_required_php_version = '5.3.0';

        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();
        theme_check_memory_limit(true);

        if (Tools::getIsset('invalid_php_version')) {
            $this->errors[] = $this->l(sprintf('You cannot use Themler because it requires PHP version %s or higher. You are running version %s. Please upgrade your PHP version to continue.',  $this->_required_php_version, $this->_php_version));
        }
        if (Tools::getIsset('no_project')) {
            $this->errors[] = $this->l(sprintf('Theme `%s` has no project file.', $this->_getThemeName()));
        }

        if ($this->ajax)
        {
            try
            {
                $themeName = $this->_getThemeName();
                $this->_checkPermissions($themeName);
                moveInnerPreview($themeName);
                $data = $this->_getThemlerData($themeName);
                echo $this->_buildPage($themeName);
            }
            catch (PermissionsException $e)
            {
                die(buildErrorsPage($e->getMessage()));
            }
        }
        else
        {
            $edit = Tools::getValue('edit', '');
            if (!empty($edit)) {
                $title = $this->l('Slider settings');
                $this->content .= $this->renderThemeSettingsForm();
            }
            else {
                $title = $this->l('List of BillionThemes');
                $this->content .= $this->renderThemesListForm();
            }
            $this->context->smarty->assign(array(
                'content' => $this->content,
                'title' => $title,
            ));
        }
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitAddconfiguration'))
        {
            $themeName = $this->_getThemeName();
            $modes = getSliderModes();
            $sliders = getSliders();

            foreach ($sliders as $sliderName => $sliderCaption)
            {
                $input = array();
                foreach ($modes as $modeName => $modeCaption) {
                    $name = getSliderSettingsName($themeName, $sliderName, $modeName);
                    $value = Tools::getValue($name, '');
                    if (Validate::isInt($value) || empty($value))
                    {
                        $value = empty($value) ? '' : $value;
                        Configuration::updateValue($name, $value);
                    }
                    else
                        $this->errors[] = $this->l('Invalid value for slider "' . $sliderCaption . '" in "' . $modeCaption . '" mode');
                }
            }

            if (!count($this->errors)) {
                $this->confirmations[] = $this->l('The settings have been updated.');
            }
        }
        return parent::postProcess();
    }

    public function setMedia()
    {
        parent::setMedia();
        if (!$this->ajax)
            $this->addCSS(__PS_BASE_URI__ . 'modules/designer/css/style.css');
    }

    private function _getThemeName()
    {
        $themeName = Tools::getValue('theme_name', '');
        if ($errors = $this->_checkErrors($themeName))
            die(Tools::jsonEncode($errors));
        return $themeName;
    }

    private function _checkThemlerRequirements($project,$themeName) 
    {
        if ($this->_php_version && version_compare($this->_php_version, $this->_required_php_version, '<')) {
            header('Location: index.php?controller=AdminAjax&token=' . $this->token . '&theme_name=' . $themeName . '&invalid_php_version=true');
        }
        if (!count((array)$project)) {
            header('Location: index.php?controller=AdminAjax&token=' . $this->token . '&theme_name=' . $themeName . '&no_project=true');
        }
    }

    private function _getThemlerData($themeName)
    {
        $project = getProject($themeName);
        $this->_checkThemlerRequirements($project, $themeName);

        $baseUrl = Designer::getBaseUrlWithLastSlash();
        $adminDir = basename(_PS_ADMIN_DIR_);
        $adminUrl = $baseUrl . $adminDir .'/index.php';

        $isNew = isset($_GET["create_theme"]) ? $_GET["create_theme"] : 'false'; // for save and publish new theme
        $isActive = (_THEME_NAME_ === $themeName || $isNew === 'true') ? 'true' : 'false';

        $data = json_encode(array(
            'themeName' => $themeName,
            'templates' => getThemeTemplates($baseUrl, $themeName),
            'themes' => $this->getThemesInfo(),
            'url' => getPSUrls($baseUrl, $themeName, $adminDir, $this->token),
            'ajax' => $adminUrl,
            'uploadImage' => $adminUrl . '?themeName='. $themeName . '&ajax=1&controller=AdminAjax&action=uploadImage&token=' . $this->token,
            'uploadTheme' => $adminUrl . '?themeName='. $themeName . '&ajax=1&controller=AdminAjax&action=uploadTheme&token=' . $this->token,
            'isActive' => $isActive,
            'isNew' => $isNew,
            'cmsVersion' => getThemeVersions(),
            'cssJsSources' => getThemeCache(getThemeDir($themeName)),
            'md5Hashes' => getThemeHashes(getThemeDir($themeName)),
            'project' => $project,
            'token' => $this->token,
            'maxRequestSize' => getMaxRequestSize()
        ));

        return $data;
    }

    public function ajaxProcessGetManifest()
    {
        try
        {
            header('Content-Type: text/cache-manifest');
            header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
            header("Cache-Control: post-check=0, pre-check=0", false);
            header("Pragma: no-cache");

            die(theme_load_manifest(Tools::getValue('ver')));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }
    
    public function ajaxProcessInitProject()
    {
        try
        {
            $themeName = Tools::getValue('themeName', '');
            $data = $this->_getThemlerData($themeName);
            die('window.pluginData = ' . $data .';');
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessExport()
    {
        try
        {
            $info = $this->_getExportInfo();
            $designer = $this->getDesignerObj();
            $response = $designer->export($info);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessSave()
    {
        try
        {
            $isNew = isset($_POST['isNew']) && $_POST['isNew'] === 'true';
            $info = $this->_getExportInfo();
            $designer = $this->getDesignerObj();
            $response = $designer->save($info, $isNew);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessPublish()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $response = $designer->publish();
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessGetTheme()
    {
        try
        {
            $designer      = isset($_REQUEST['data']['id']) ? new DesignerTabSelf($_REQUEST['data']['id']) : $this->getDesignerObj(); // download by id
            $userThemeName = isset($_REQUEST['data']['themeName']) ? $_REQUEST['data']['themeName'] : '';
            $includeEditor = !isset($_REQUEST['data']['includeEditor']) || $_REQUEST['data']['includeEditor'] === 'true';
            $response      = $designer->getTheme($userThemeName, $includeEditor);
            die($response);
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessRequest()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $newName  = isset($_POST['newName']) ? $_POST['newName'] : '';
            $method   = isset($_POST['method']) ? $_POST['method'] : '';
            if (method_exists($designer, $method))
            {
                switch ($method) {
                    case 'update':
                        $response = $designer->$method();
                        break;
                    default:
                        $response = $designer->$method($newName);
                        break;
                }
                die(Tools::jsonEncode($response));
            }
            die(Tools::jsonEncode(array('result' => 'error', 'message' => "Method $method doesn't exist")));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessGetFiles()
    {
        try
        {
            $mask     = isset($_POST['mask']) ? $_POST['mask'] : null;
            $filter   = isset($_POST['filter']) ? $_POST['filter'] : null;
            $designer = $this->getDesignerObj();
            $response = $designer->getFiles($mask, $filter);
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }
    
    public function ajaxProcessSetFiles()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $files = array();

            $response = null;
            $data = isset($_POST['files']) ? $_POST['files'] : false;
            if ($data) {
                $files = json_decode($data, true);
                if (null === $files) {
                    trigger_error('Data is incorrect in setFiles method: "' . $data . '"', E_USER_ERROR);
                }
                $response = array('result' => 'OK');
            }
            else
            {
                $chunk = new Chunk();

                if (!$chunk->save($this->_getChunkInfo())) {
                    header($_SERVER['SERVER_PROTOCOL'] . ' 400 Bad Request', true, 400);
                    return array();
                }

                if ($chunk->last())
                {
                    $files = json_decode($chunk->complete(), true);
                    $response = array('result' => 'done');
                }
                else 
                {
                    $response = array('result' => 'processed');
                }
            }

            if ($files && count($files)) {
                $designer->setFiles($files);
            }
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }
    
    public function ajaxProcessUploadImage()
    {
        $designer = $this->getDesignerObj();
        $response = $designer->uploadFile();
        die(Tools::jsonEncode($response));
    }

    public function ajaxProcessUploadTheme()
    {
        $designer = $this->getDesignerObj();
        $response = $designer->uploadTheme();
        die(Tools::jsonEncode($response));
    }

    public function ajaxProcessClear()
    {
        try
        {
            $designer = $this->getDesignerObj();
            $response = $designer->chunkClear();
            die(Tools::jsonEncode($response));
        }
        catch (PermissionsException $e)
        {
            die($this->_exceptionResponse($e));
        }
    }

    public function ajaxProcessReloadTemplatesInfo()
    {
        $baseUrl = Designer::getBaseUrlWithLastSlash();
        $themeName = Tools::getValue('themeName', '');
        $response = array('templates' => getThemeTemplates($baseUrl, $themeName));
        die(Tools::jsonEncode($response));
    }
    
    public function ajaxProcessReloadThemesInfo()
    {
        $response = $this->getThemesInfo();
        die(Tools::jsonEncode($response));
    }

    private function _getExportInfo()
    {
        $data = isset($_POST['info']) ? $_POST['info'] : false;
        if ($data === false) {
            $info = $this->_getChunkInfo();
        } else {
            $info = json_decode($data, true);
        }
        if (null === $info) {
            trigger_error('Data is incorrect: "' . $data . '"', E_USER_ERROR);
        }
        return $info;
    }

    private function _getChunkInfo()
    {
        return array(
            'id' => isset($_POST['id']) ? $_POST['id'] : '',
            'content' => isset($_POST['content']) ? $_POST['content'] : '',
            'current' => isset($_POST['current']) ? $_POST['current'] : '',
            'total' => isset($_POST['total']) ? $_POST['total'] : '',
            'encode' => !empty($_POST['encode']),
            'blob' => !empty($_POST['blob'])
        );
    }

    private function _buildPage($themeName)
    {
        $version = theme_get_manifest_version($themeName);
        theme_copy_manifest($themeName, $version);
        $manifestVersion = '&ver=' . $version;

        $token = $this->token;
        $baseUrl = Designer::getBaseUrlWithLastSlash();
        $adminUrl = $baseUrl . basename(_PS_ADMIN_DIR_) .'/index.php';
        $moduleUrl = $baseUrl . 'modules/designer';

        $manifestPath = FilesHelper::normalizePath(getThemeDir($themeName)) . '/export/themler.manifest';
        $manifest = file_exists($manifestPath) ? sprintf('manifest="%s/Export/Manifest.php"', $moduleUrl) : '';
        $version = time();

        $output = <<<EOT
<!DOCTYPE html>
<html manifest="$adminUrl?ajax=1&action=getManifest&controller=AdminAjax&token=$token$manifestVersion">
<head>
<title>Themler</title>
<meta http-equiv="X-UA-Compatible" content="IE=edge" />

<script type="text/javascript" src="$adminUrl?ajax=1&action=initProject&controller=AdminAjax&token=$token&themeName=$themeName"></script>
<script type="text/javascript" src="$moduleUrl/DataProvider.js?version=$version"></script>
<script type="text/javascript" src="$moduleUrl/loader.js?version=$version"></script>

</head>
<body>

</body>
</html>
EOT;
        return $output;
    }
    
    private function getDesignerObj()
    {
        theme_check_memory_limit();
        $themeName = Tools::getValue('themeName', '');
        if ($errors = $this->_checkErrors($themeName)) {
            die(Tools::jsonEncode($errors));
        }

        return new DesignerTabSelf($themeName);
    }

    private function getThemesInfo()
    {
        $baseUrl = Designer::getBaseUrlWithLastSlash();
        $adminUrl = $baseUrl . basename(_PS_ADMIN_DIR_) . '/';
        $themesUrl = $baseUrl . 'themes';
        $shop = $this->context->shop;

        $themes = array();
        foreach (Theme::getAvailable(false) as $themeName)
        {
            if (strpos($themeName, _PREVIEW_SUFFIX_) !== FALSE ||
                !file_exists(getProjectPath($themeName))) continue;

            $id = Designer::getThemeId($themeName);
            $themeUrl = $themesUrl . '/' . $themeName;
            $url = self::$currentIndex . '&token=' . $this->token . '&theme_name=' . $themeName;
            $themes['themes'][$themeName] = array(
                'themeName' => $themeName,
                'thumbnailUrl' => $themeUrl. '/preview.jpg',
                'openUrl' => $adminUrl . $url . '&ajax=1&ver=' . theme_get_manifest_version($themeName) . getDesktopParams(),
                'settingsUrl' => $url . '&edit=1',
                'isActive' => $id && $shop->id_theme == $id,
            );
        }
        return $themes;
    }

    private function renderThemesListForm()
    {
        $themes = $this->getThemesInfo();

        $result = '';
        foreach ($themes['themes'] as $id => $theme)
        {
            $result .= '<div class="theme-item">'
                    . '<div class="item-title"><h4>'.$theme['themeName'].'</h4></div>'
                    . '<div class="item-view"><img src="'.$theme['thumbnailUrl']. '" alt="'.$theme['themeName'].'"></div>';
            
            $result .= '<div class="item-actions"><a href="' . $theme['openUrl'] .'">Customize with Themler</a></div>';
            $result .= '<div class="item-actions"><a href="' . $theme['settingsUrl'] . '">Theme Settings</a></div>';
            $result .= '</div>';
        }
        $output = <<<EOF
<div class="theme-list">
<div id="theme-editor-list">{$result}</div>
EOF;
        return $output;
    }

    private function renderThemeSettingsForm()
    {
        $themeName = $this->_getThemeName();
        $modes = getSliderModes();
        $sliders = getSliders();

        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        $defaultSettings = getSliderDefaultSettings($themeDir);

        $index = 0;
        foreach ($sliders as $sliderName => $sliderCaption) {
            $input = array();
            foreach ($modes as $modeName => $modeCaption) {
                $options = array(
                    array(
                    'id_option' => '',
                    'name' => '',
                    )
                );
                for ($i=1; $i<=8; $i++)
                {
                    $r = 24 % $i;
                    if ($r) continue;
                    $options[] = array(
                        'id_option' => 24 / $i,
                        'name' => $i,
                    );
                };
                $name = getSliderSettingsName($themeName, $sliderName, $modeName);
                $value = Configuration::get($name);
                if ($modeName === 'itemsInRow') {
                    $dv = 2;
                    $input[] = array('type' => 'text', 'label' => $this->l($modeCaption), 'name' => $name, 'class' => ' fixed-width-xl');
                }
                else {
                    $dv = '';
                    $input[] = array('type' => 'select', 'label' => $this->l($modeCaption), 'name' => $name, 'options' => array('query' => $options, 'id' => 'id_option', 'name' => 'name'));
                }
                $defaultValue = isset($defaultSettings) && !empty($defaultSettings[$modeName]) ? $defaultSettings[$modeName] : $dv;
                $hasEmptyRecord = is_null($value); // has database record with null value
                $this->fields_value[$name] = $hasEmptyRecord ? $dv : ($value ? $value : $defaultValue);
            }
            $this->fields_form[$index]['form'] = array(
                'legend' => array(
                    'title' => $this->l($sliderCaption),
                    'icon' => 'icon-edit'
                ),
                'input' => $input,
            );
            $index++;
        }

        $this->fields_form[$index]['form'] = array(
            'input' => array(
                array(
                    'type' => 'hidden',
                    'name' => 'theme_name',
                    'value' => $themeName
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'id' => 'submitSliderSettings',
                'icon' => 'process-icon-save'
            )
        );

        $this->fields_form = array_values($this->fields_form);
        $this->multiple_fieldsets = true;
        return parent::renderForm();
    }

    private function _exceptionResponse($e)
    {
        return Tools::jsonEncode(array(
                'result' => 'fail',
                'type' => 'permission',
                'error' => $e->getMessage()
               ));
    }

    private function _checkErrors($themeName = '')
    {
        $themeName = empty($themeName) ? Tools::getValue('themeName', null) : $themeName;
        if (is_null($themeName)) {
            return array(
                'result' => 'fail',
                'type'   => 'themeName',
                'title'  => 'Empty theme name',
                'error'  => 'Empty theme name is invalid'
            );
        }

        return null;
    }

    private function _checkPermissions($themeName)
    {
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        $previewDir = $themeDir . _PREVIEW_SUFFIX_;
        $dirs = array(_PS_OVERRIDE_DIR_, _PS_DOWNLOAD_DIR_, _PS_MODULE_DIR_, _PS_UPLOAD_DIR_, $themeDir, $previewDir);
        foreach ($dirs as $dir) {
            $rootOnly = strpos($dir, '/themes/') === FALSE; // check themes subdirectories only
            $check = FilesHelper::checkFiles($dir, true, true, $rootOnly);
        }

        return true;
    }
    
    /* Methods to support Theme installation in Themler */
}