<?php

include_once(dirname(__FILE__).'/../designerpreview/defines.inc.php');
include_once(dirname(__FILE__).'/Export/EditorHelper.php');
include_once(dirname(__FILE__).'/Export/FilesHelper.php');

class Designer extends Module
{
    public function __construct()
    {
        $this->name = 'designer';
        $this->tab = 'administration';
        $this->version = '1.0';
        
        parent::__construct();

        $this->displayName = $this->l('Billion Themler');
        $this->description = $this->l('Billion Themler');
        $this->confirmUninstall = $this->l('Are you sure you want to delete your details ?');		
    }

    public function install()
    {
        return parent::install() && $this->installAdminTab();
    }
    
    private function installAdminTab()
    {
        $tab             = new Tab();
        $parentTab       = new Tab();
        $parentTab->name = array();
        $tab->name       = array();
        foreach (Language::getLanguages(true) as $lang) {
            $parentTab->name[$lang['id_lang']] = 'Billion Themler';
            $tab->name[$lang['id_lang']] = 'Billion Themler';
        }

        $parentTab->class_name = 'DesignerTab';
        $parentTab->module     = $this->name;
        $parentTab->active     = 1;
        $parentTab->id_parent  = 0;
        $parentTab->add();

        $id_parent_tab         = (int)Tab::getIdFromClassName('DesignerTab');
        $tab->class_name       = 'AdminAjax';
        $tab->module           = $this->name;
        $tab->active           = 1;
        $tab->id_parent        = $id_parent_tab;
        return $tab->add($this->id);
    }

    public function uninstall()
    {
        return parent::uninstall() && $this->uninstallAdminTab();
    }
    
    private function uninstallAdminTab()
    {
        $parentTab = new Tab(Tab::getIdFromClassName('DesignerTab'));
        $tab = new Tab(Tab::getIdFromClassName('AdminAjax'));
        return $tab->delete() && $parentTab->delete();
    }

    public function getContent()
    {
        $context = Context::getContext();
        $token = Tools::getAdminToken('AdminAjax'.(int)Tab::getIdFromClassName('AdminAjax').(int)$context->employee->id);
        header('Location: index.php?controller=AdminAjax&token=' . $token);
        exit;
    }		
    
    /**
      * Return employee instance from its id
      *
      * @param string $id_employee employee id
      * @return Employee instance
      */
    public static function getEmployeeById($id_employee)
    {
        if (!Validate::isUnsignedId($id_employee))
            die (Tools::displayError());

        $result = Db::getInstance()->getRow('
        SELECT *
        FROM `'._DB_PREFIX_.'employee`
        WHERE `active` = 1
        AND `id_employee` = '.(int)$id_employee);
        if (!$result)
            return false;
        
        $employee = new Employee();
        $employee->id = $result['id_employee'];
        $employee->id_profile = $result['id_profile'];
        foreach ($result as $key => $value)
            if (property_exists($employee, $key))
                $employee->{$key} = $value;
        return $employee;
    }

    public static function getFirstCategory()
    {
        $shop = Designer::getShop();
        $default_category = $shop->getCategory();
        $sql = 'SELECT MIN(`id_category`) AS id_category
            FROM `'._DB_PREFIX_.'category`
            WHERE `id_parent` = '.(int)$default_category;
        return Designer::selectChild($sql);
    }

    public static function getFirstProduct()
    {
        $sql = 'SELECT MIN(`id_product`) AS id_product FROM `'._DB_PREFIX_.'product`';
        return Designer::selectChild($sql);
    }

    public static function getFirstCMSPage()
    {
        $sql = 'SELECT MIN(`id_cms`) AS id_cms FROM `'._DB_PREFIX_.'cms`';
        return Designer::selectChild($sql);
    }
    
    public static function selectChild($sql)
    {
        $result = 0;
        if ($id_child = Db::getInstance()->getValue($sql))
            $result = (int)$id_child;
        return $result;
    }
    
    public static function getBaseUrlWithLastSlash()
    {
        $baseUrl = (Tools::usingSecureMode() ? _PS_BASE_URL_SSL_ : _PS_BASE_URL_) . __PS_BASE_URI__;
        return $baseUrl . (substr($baseUrl, -1) !== '/' ? '/' : '');
    }
    
    public static function getDownloadUrl()
    {
        return self::getBaseUrlWithLastSlash() . 'download';
    }
    
    /*
     * Method generates a random 32 character string consisting of, by default, only A-Z, a-z, and 0-9
     */
    public static function generateThemeName()
    {
        $pattern = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
        $length = 10; // themeName length
        for ($s = '', $i = 0, $z = strlen($pattern)-1; $i < $length; $x = rand(0,$z), $s .= $pattern{$x}, $i++);
        return $s;
    }

    public static function getShop()
    {
        $context = Context::getContext();
        return $context->shop;
    }

    public static function isActiveTheme($themeName)
    {
        $id_theme = Designer::getThemeId($themeName);
        if (!$id_theme)
            return false;

        $shop = Designer::getShop();
        return $shop->id_theme == $id_theme;
    }

    public static function setActiveThemeForShop($themeName)
    {
        $id_theme = Designer::getThemeId($themeName);
        if (!$id_theme) 
            $id_theme = Designer::addTheme($themeName);
        
        $shop = Designer::getShop();
        if ($id_theme && $shop->id_theme != $id_theme)
        {
            $shop->id_theme = $id_theme;
            $shop->update();
        }
        
        return $id_theme;
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function getThemeId($themeDir)
    {
        $result = 0;
        $sql = 'SELECT id_theme FROM `'._DB_PREFIX_.'theme`
            WHERE `directory` = \''.(string)$themeDir .'\'';
        if ($id_theme = Db::getInstance()->getValue($sql))
            $result = (int)$id_theme;
        return $result;
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function addTheme($themeName)
    {
        $sql = 'INSERT INTO `'._DB_PREFIX_.'theme` (`name`, `directory`) VALUES (\''.$themeName.'\', \''.$themeName.'\')';
        Db::getInstance()->execute($sql);
        return Db::getInstance()->Insert_ID();
    }
    
    /*
     * Retrieves theme ID by name from database
     */
    public static function deleteTheme($themeName)
    {
        $sql = 'DELETE FROM `'._DB_PREFIX_.'theme` WHERE name = \''.$themeName.'\'';
        if (!Db::getInstance()->execute($sql))
            return false;
    }
}